% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/piece_mesh-rayvertex.R
\name{piece_mesh}
\alias{piece_mesh}
\title{Create rayvertex objects}
\usage{
piece_mesh(
  piece_side = "tile_back",
  suit = NA,
  rank = NA,
  cfg = pp_cfg(),
  x = 0,
  y = 0,
  z = NA,
  angle = 0,
  axis_x = 0,
  axis_y = 0,
  width = NA,
  height = NA,
  depth = NA,
  envir = NULL,
  ...,
  scale = 1,
  res = 72
)
}
\arguments{
\item{piece_side}{A string with piece and side separated by a underscore e.g. "coin_face"}

\item{suit}{Number of suit (starting from 1).}

\item{rank}{Number of rank (starting from 1)}

\item{cfg}{Piecepack configuration list or \code{pp_cfg} object,
a list of \code{pp_cfg} objects,
or a character vector referring to names in \code{envir}
or a character vector referring to object names that
can be retrieved by \code{base::dynGet()}.}

\item{x}{Where to place piece on x axis of viewport}

\item{y}{Where to place piece on y axis of viewport}

\item{z}{z-coordinate of the piece.  Has no effect if \code{op_scale} is \code{0}.}

\item{angle}{Angle (on xy plane) to draw piece at}

\item{axis_x}{First coordinate of the axis unit vector.}

\item{axis_y}{Second coordinate of the axis unit vector.}

\item{width}{Width of piece}

\item{height}{Height of piece}

\item{depth}{Depth (thickness) of piece.  Has no effect if \code{op_scale} is \code{0}.}

\item{envir}{Environment (or named list) containing configuration list(s).}

\item{...}{Ignored.}

\item{scale}{Multiplicative scaling factor to apply to width, height, and depth.}

\item{res}{Resolution of the faces.}
}
\value{
A rayvertex object.
}
\description{
\code{piece_mesh()} creates 3d board game piece objects for use with the rayvertex package.
}
\examples{
  \donttest{
    if (require("rayvertex")) {
        cfg <- game_systems("sans3d")$piecepack
        rs <- function(shape) {
             rasterize_scene(shape, light_info = directional_light(c(0, 0, 1)))
        }
        rs(piece_mesh("tile_face", suit = 3, rank = 3, cfg = cfg))
        rs(piece_mesh("coin_back", suit = 4, rank = 2, cfg = cfg))
        rs(piece_mesh("pawn_face", suit = 1, cfg = cfg))
    }
  }
}
\seealso{
See \url{https://www.rayvertex.com} for more information about the \code{rayvertex} package.
See \code{\link{geometry_utils}} for a discussion of the 3D rotation parameterization.
}
